use anyhow::Result;
use clap::Parser;
use std::{
    io::{Read, Write},
    net::TcpStream,
};

#[derive(Parser, Debug)]
#[command(author, version)]
pub struct Opts {
    #[arg(long, name = "port", short = 'p')]
    port: usize,
}

/// Height (in pixels) of the framebuffer.
const HEIGHT: usize = 1404;
/// Width (in pixels) of the framebuffer.
const WIDTH: usize = 1872;
/// How many bytes represent one pixel in the framebuffer.

const IN_SIZE: usize = WIDTH * HEIGHT / 4;
const OUT_SIZE: usize = WIDTH * HEIGHT * 3;

fn main() -> Result<()> {
    println!("XX");

    let mut stream = TcpStream::connect("10.143.0.112:1234")?;
    let mut stdout = std::io::stdout();

    let mut in_buf = [0u8; IN_SIZE];
    let mut out_buf = Vec::with_capacity(OUT_SIZE);

    let mask = 0b00000011;

    loop {
        out_buf.clear();

        stream.read_exact(&mut in_buf)?;
        for i in &in_buf {
            let a = (i >> 6) & mask;
            let b = (i >> 4) & mask;
            let c = (i >> 2) & mask;
            let d = (i >> 0) & mask;
            let pix = [a, b, c, d];

            for p in pix {
                // black 00
                // red
                // green
                // white 11
                match p {
                    0b00 => {
                        out_buf.push(0u8);
                        out_buf.push(0u8);
                        out_buf.push(0u8);
                    }

                    0b01 => {
                        out_buf.push(255u8);
                        out_buf.push(0u8);
                        out_buf.push(0u8);
                    }

                    0b10 => {
                        out_buf.push(0u8);
                        out_buf.push(255u8);
                        out_buf.push(0u8);
                    }

                    0b11 => {
                        out_buf.push(255u8);
                        out_buf.push(255u8);
                        out_buf.push(255u8);
                    }

                    _ => unreachable!(),
                }
            }
        }

        stdout.write_all(&out_buf[..])?;
    }
}
